@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -38,6 +38,7 @@
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include <gcu/element.h>
 #include <pango/pango-attributes.h>
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <list>
 
@@ -113,6 +114,10 @@ bool gcpFragment::OnChanged (bool save)
 			m_BeginAtom = m_StartSel;
 			m_EndAtom = CurPos;
 		}
+	} else if (m_EndSel <= m_BeginAtom) {
+		int delta = CurPos - m_EndSel;
+		m_BeginAtom += delta;
+		m_EndAtom += delta;
 	} else if ((m_EndAtom <= m_EndSel && m_EndAtom >= m_StartSel) ||
 		(m_BeginAtom <= m_EndSel && m_BeginAtom >= m_StartSel) ||
 		(m_BeginAtom + 3 >= CurPos)) {
@@ -189,12 +194,22 @@ void gcpFragment::Add (GtkWidget* w)
 		pango_layout_get_extents (m_Layout, &rect, NULL);
 		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 		pango_layout_set_text (m_Layout, m_buf.c_str (), -1);
-		m_buf.clear ();
 		if (m_AttrList) {
 			pango_layout_set_attributes (m_Layout, m_AttrList);
 			pango_attr_list_unref (m_AttrList);
 			m_AttrList = NULL;
 		}
+		if (m_buf.length () > 0) {
+			m_buf.clear ();
+			pango_layout_index_to_pos (m_Layout, m_BeginAtom, &rect);
+			m_lbearing = rect.x / PANGO_SCALE;
+			pango_layout_index_to_pos (m_Layout, m_EndAtom, &rect);
+			m_lbearing += rect.x / PANGO_SCALE;
+			m_lbearing /=  2;
+			iter = pango_layout_get_iter (m_Layout);
+			m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+			pango_layout_iter_free (iter);
+		}
 		pango_layout_get_extents (m_Layout, NULL, &rect);
 		m_length = rect.width / PANGO_SCALE;
 		m_height = rect.height / PANGO_SCALE;
@@ -241,24 +256,24 @@ void gcpFragment::Add (GtkWidget* w)
 		switch (align) {
 		case -2:
 			x -= pTheme->GetChargeSignSize () / 2.;
-			y += pTheme->GetChargeSignHeight () / 2.;
-			break;
-		case -1:
-			x -= pTheme->GetChargeSignSize () + 2.;
-			break;
-		case 0:
-		case -3:
-			x -= pTheme->GetChargeSignSize () / 2.;
-			break;
-		case 1:
-			x += 2.;
+			y += pTheme->GetChargeSignSize () / 2.;
 			break;
+			case -1:
+				x -= pTheme->GetChargeSignSize () + pTheme->GetPadding ();
+				break;
+			case 0:
+			case -3:
+				x -= pTheme->GetChargeSignSize () / 2.;
+				break;
+			case 1:
+				x += pTheme->GetPadding ();
+				break;
 		case 2:
 			x -= pTheme->GetChargeSignSize () / 2.;
-			y -= pTheme->GetChargeSignHeight () / 2.;
+			y -= pTheme->GetChargeSignSize () / 2.;
 			break;
 		}
-		y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
+		y -= pTheme->GetChargeSignSize () / 2.;
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -375,21 +390,21 @@ void gcpFragment::Update (GtkWidget* w)
 				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				x -= pTheme->GetChargeSignSize () + 2.;
+				x -= pTheme->GetChargeSignSize () + pTheme->GetPadding ();
 				break;
 			case 0:
 			case -3:
 				x -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			case 1:
-				x += 2.;
+				x += pTheme->GetPadding ();
 				break;
 			case 2:
 				x -= pTheme->GetChargeSignSize () / 2.;
 				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
-			y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
+			y -= pTheme->GetChargeSignSize () / 2.;
 			item = g_object_get_data (G_OBJECT (group), "circle");
 			g_object_set (G_OBJECT (item),
 						"x1", x,
@@ -434,21 +449,21 @@ void gcpFragment::Update (GtkWidget* w)
 				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				x -= pTheme->GetChargeSignSize () + 2.;
+				x -= pTheme->GetChargeSignSize () + pTheme->GetPadding ();
 				break;
 			case 0:
 			case -3:
 				x -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			case 1:
-				x += 2.;
+				x += pTheme->GetPadding ();
 				break;
 			case 2:
 				x -= pTheme->GetChargeSignSize () / 2.;
 				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
-			y -= pTheme->GetChargeSignHeight () / 2. - pTheme->GetChargeYAlign () + pTheme->GetChargeSignSize () - 1.;
+			y -= pTheme->GetChargeSignSize () / 2.;
 			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 						group,
 						gnome_canvas_group_ext_get_type (),
@@ -642,7 +657,7 @@ bool gcpFragment::Load (xmlNodePtr node)
 			m_BeginAtom = m_buf.length ();
 			m_buf += m_Atom->GetSymbol();
 			m_Atom->SetCoords (m_x, m_y);
-			m_EndAtom = m_buf.length () - m_BeginAtom;
+			m_EndAtom = m_buf.length ();
 		} else if (!strcmp ((const char*) child->name, "charge")) {
 			int start = m_buf.length (), end;
 			tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
