@@ -34,6 +34,7 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gprintable.h"
+#include <glib/gi18n-lib.h>
 #include <stdexcept>
 
 static void on_text_changed (gcpText *text)
@@ -668,6 +669,11 @@ bool gcpText::OnChanged (bool save)
 		pData->Items.erase (this);
 		return false;
 	}
+	if (strlen (pango_layout_get_text (m_Layout))) {
+		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
+		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
+	}
 	PangoRectangle rect;
 	pango_layout_get_extents (m_Layout, NULL, &rect);
 	m_length = rect.width / PANGO_SCALE;
