@@ -2,7 +2,7 @@
  * GChemPaint library
  * window.cc
  *
- * Copyright (C) 2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2006-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -23,15 +23,18 @@
 #include "gchempaint-config.h"
 #include "application.h"
 #include "document.h"
+#include "newfiledlg.h"
 #include "window.h"
 #include "about.h"
 #include "preferences.h"
+#include "theme.h"
 #include "zoomdlg.h"
 #include <gcu/filechooser.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
+#include <glib/gi18n-lib.h>
 
 static void on_destroy (GtkWidget* widget, gcpWindow* Win)
 {
@@ -39,11 +42,21 @@ static void on_destroy (GtkWidget* widget, gcpWindow* Win)
 	delete Win;
 }
 
+static bool on_delete_event (GtkWidget* widget, GdkEvent *event, gcpWindow* Win)
+{
+	return !Win->VerifySaved ();
+}
+
 static void on_file_new(GtkWidget* widget, gcpWindow* Win)
 {
 	Win->OnFileNew();
 }
 
+static void on_themed_file_new(GtkWidget* widget, gcpWindow* Win)
+{
+	new gcpNewFileDlg (Win->GetApplication ());
+}
+
 static void on_file_open(GtkWidget* widget, gcpWindow* Win)
 {
 	Win->OnFileOpen();
@@ -192,21 +205,21 @@ static bool on_focus_in (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win
 	gcpApplication *App = Win->GetApplication ();
 	App->NotifyFocus (true, Win);
 	gtk_clipboard_request_contents (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
-	return false;
+	return true;
 }
 
 static bool on_focus_out (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
 {
 	Win->GetDocument ()->GetView ()->ShowCursor (false);
 	Win->GetApplication ()->NotifyFocus (false);
-	return false;
+	return true;
 }
 
 static bool on_state (GtkWidget *widget, GdkEventWindowState *event, gcpWindow* Win)
 {
 	if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
 		Win->GetApplication ()->NotifyIconification (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED);
-	return false;
+	return true;
 }
 
 static void on_help (GtkWidget *widget, gcpWindow* Win)
@@ -283,7 +296,7 @@ static void on_recent (GtkRecentChooser *widget, gcpWindow *Win)
 	gcpApplication *App = Win->GetApplication ();
 	GtkRecentInfo *info = gtk_recent_chooser_get_current_item (widget);
 	gcpDocument *pDoc = Win->GetDocument ();
-	App->FileProcess (gtk_recent_info_get_uri (info), gtk_recent_info_get_mime_type (info), false, NULL, (!pDoc->HasChildren () && !pDoc->IsDirty ())? pDoc: NULL);
+	App->FileProcess (gtk_recent_info_get_uri (info), gtk_recent_info_get_mime_type (info), false, NULL, (!pDoc->HasChildren () && !pDoc->GetDirty ())? pDoc: NULL);
 	gtk_recent_info_unref (info);
 }
 
@@ -294,6 +307,8 @@ static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
 	  { "New", GTK_STOCK_NEW, N_("_New File"), NULL,
 		  N_("Create a new file"), G_CALLBACK (on_file_new) },
+	  { "NewThemed", GTK_STOCK_NEW, N_("_New File with Theme..."), "<shift><control>N",
+		  N_("Create a new file using a theme"), G_CALLBACK (on_themed_file_new) },
 	  { "Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
 		  N_("Open a file"), G_CALLBACK (on_file_open) },
 	  { "Save", GTK_STOCK_SAVE, N_("_Save"), "<control>S",
@@ -370,6 +385,7 @@ static const char *ui_description =
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
 "      <menuitem action='New'/>"
+"      <menuitem action='NewThemed'/>"
 "      <menuitem action='Open'/>"
 "      <menuitem action='Save'/>"
 "      <menuitem action='SaveAs'/>"
@@ -455,7 +471,7 @@ static const char *ui_web_description =
 "  </menubar>"
 "</ui>";
 
-gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
+gcpWindow::gcpWindow (gcpApplication *App, char const *Theme, char const *extra_ui)
 {
 	GtkWidget *vbox;
 	GtkWidget *bar;
@@ -468,6 +484,7 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 	g_object_set (G_OBJECT (m_Window), "urgency-hint", false, NULL);
 	g_object_set_data (G_OBJECT (m_Window), "gcp-role", (void*) 1);
 	g_signal_connect (G_OBJECT (m_Window), "destroy", G_CALLBACK (on_destroy), this);
+	g_signal_connect (G_OBJECT (m_Window), "delete-event", G_CALLBACK (on_delete_event), this);
 	g_signal_connect (G_OBJECT (m_Window), "focus_in_event", G_CALLBACK (on_focus_in), this);
 	g_signal_connect (G_OBJECT (m_Window), "focus_out_event", G_CALLBACK (on_focus_out), this);
 	g_signal_connect (G_OBJECT (m_Window), "window-state-event", G_CALLBACK (on_state), this);
@@ -528,6 +545,8 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 	gtk_toolbar_set_tooltips(GTK_TOOLBAR(bar), true);
 	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
 	m_Doc = new gcpDocument (App, true, this);
+	if (Theme)
+		m_Doc->SetTheme (ThemeManager.GetTheme (Theme));
 	gtk_window_set_title (m_Window, m_Doc->GetTitle ());
 	w = m_Doc->GetView ()->CreateNewWidget ();
 	GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
@@ -573,7 +592,7 @@ void gcpWindow::OnSave()
 
 void gcpWindow::OnFileOpen()
 {
-	FileChooser (m_App, false, m_App->GetSupportedMimeTypes (), (!m_Doc->HasChildren () && !m_Doc->IsDirty ())? m_Doc: NULL);
+	FileChooser (m_App, false, m_App->GetSupportedMimeTypes (), (!m_Doc->HasChildren () && !m_Doc->GetDirty ())? m_Doc: NULL);
 }
 
 void gcpWindow::OnProperties()
@@ -627,10 +646,6 @@ void gcpWindow::SetActive (gcpDocument* pDoc, GtkWidget* w)
 {
 }
 
-void gcpWindow::OnFileClose ()
-{
-}
-
 void gcpWindow::OnUndo()
 {
 	m_Doc->OnUndo();
@@ -689,11 +704,11 @@ void gcpWindow::Zoom (double zoom)
 	if (zoom >= 0.2 && zoom <= 8.)
 		pView->Zoom (zoom);
 	else {
-		Dialog *pDialog = GetApplication ()->GetDialog ("Zoom");
+		Dialog *pDialog = m_Doc->GetDialog ("Zoom");
 		if (pDialog)
 			gtk_window_present (pDialog->GetWindow ()); 
 		else
-			new gcpZoomDlg (GetApplication ());
+			new gcpZoomDlg (m_Doc);
 	}
 }
 
@@ -741,10 +756,14 @@ bool gcpWindow::OnKeyReleased(GtkWidget* widget, GdkEventKey* ev)
 		false;
 }
 
-void gcpWindow::Close ()
+bool gcpWindow::Close ()
 {
-	m_Doc->GetView ()->PrepareUnselect ();
-	gtk_widget_destroy (GTK_WIDGET (m_Window));
+	if (VerifySaved ()) {
+		m_Doc->GetView ()->PrepareUnselect ();
+		gtk_widget_destroy (GTK_WIDGET (m_Window));
+		return true;
+	}
+	return false;
 }
 
 
@@ -759,3 +778,24 @@ void gcpWindow::ActivateActionWidget (char const *path, bool activate)
 	if (w)
 		gtk_widget_set_sensitive (w, activate);
 }
+
+bool gcpWindow::VerifySaved ()
+{
+	if (!m_Doc->GetDirty ())
+		return true;
+	gchar* str = g_strdup_printf(_("\"%s\" has been modified.  Do you wish to save it?"), m_Doc->GetTitle ());
+	GtkWidget* mbox;
+	int res;
+	do {
+		mbox = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, str);
+		gtk_dialog_add_button (GTK_DIALOG (mbox),  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+		res = gtk_dialog_run (GTK_DIALOG (mbox));
+		gtk_widget_destroy (mbox);
+		if (res == GTK_RESPONSE_YES)
+			OnSave ();
+	} while ((res == GTK_RESPONSE_YES) && (m_Doc->GetFileName () == NULL));
+	if (res == GTK_RESPONSE_NO)
+		m_Doc->SetDirty (false);
+	g_free(str);
+	return (res != GTK_RESPONSE_CANCEL);
+}
