@@ -4,7 +4,7 @@
  * GChemPaint arrows plugin
  * retrosynthesis.cc
  *
- * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,12 +22,15 @@
  * USA
  */
 
+#include "gchempaint-config.h"
 #include "retrosynthesis.h"
 #include "retrosynthesisarrow.h"
 #include "retrosynthesisstep.h"
 #include "lib/document.h"
+#include "lib/theme.h"
 #include "lib/widgetdata.h"
 #include "lib/view.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 
 TypeId RetrosynthesisType = NoType;
@@ -48,7 +51,7 @@ gcpRetrosynthesis::~gcpRetrosynthesis ()
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
 	gcpOperation *pOp = pDoc->GetCurrentOperation ();
 	while ((pObj = GetFirstChild (i))) {
-		if (pObj->GetType () == RetrosynthesisArrowType) {
+		if ((pObj->GetType () == RetrosynthesisArrowType)) {
 			arrow = reinterpret_cast<gcpRetrosynthesisArrow *> (pObj);
 			arrow->SetStartStep (NULL);
 			arrow->SetEndStep (NULL);
@@ -120,17 +123,6 @@ bool gcpRetrosynthesis::Load (xmlNodePtr node)
 	return true;
 }
 
-void gcpRetrosynthesis::Add (GtkWidget* w)
-{
-	map<string, Object*>::iterator i;
-	Object* p = GetFirstChild (i);
-	while (p)
-	{
-		p->Add (w);
-		p = GetNextChild (i);
-	}
-}
-
 typedef struct
 {
 	double x, y;
@@ -141,6 +133,7 @@ typedef struct
 bool gcpRetrosynthesis::Build (list<Object*>& Children) throw (invalid_argument)
 {
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpWidgetData  *pData= reinterpret_cast<gcpWidgetData *> (g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data"));
 	list<Object*>::iterator i, iend = Children.end ();
 	map<Object *, ObjectData> Objects;
@@ -156,7 +149,7 @@ bool gcpRetrosynthesis::Build (list<Object*>& Children) throw (invalid_argument)
 		if (Id == MoleculeType) {
 			pData->GetObjectBounds (*i, &od.r);
 			od.x = (od.r.x0 + od.r.x1) / 2.;
-			od.y = (*i)->GetYAlign () * pData->ZoomFactor;
+			od.y = (*i)->GetYAlign () * pTheme->GetZoomFactor ();
 			od.step = new gcpRetrosynthesisStep (this, reinterpret_cast<gcpMolecule *>(*i));
 			nmol++;
 			Objects[*i] = od;
@@ -179,12 +172,12 @@ bool gcpRetrosynthesis::Build (list<Object*>& Children) throw (invalid_argument)
 		// x1, y1 will now be the coordinates of a normalized vector:
 		x1 -= x0;
 		y1 -= y0;
-		x0 *= pData->ZoomFactor;
-		y0 *= pData->ZoomFactor;
+		x0 *= pTheme->GetZoomFactor ();
+		y0 *= pTheme->GetZoomFactor ();
 		l = sqrt (x1 * x1 + y1 * y1);
 		x1 /= l;
 		y1 /= l;
-		l *= pData->ZoomFactor; // half length of the arrow on the screen
+		l *= pTheme->GetZoomFactor (); // half length of the arrow on the screen
 		// No molecule should be nearer than that
 		minright = minleft = DBL_MAX;
 		Left = Right = NULL;
@@ -313,6 +306,7 @@ static void AlignStep (map<Object*, StepData> &data, gcpRetrosynthesisStep *step
 	double x0, y0, x1, y1, x, y, l, dx, dy;
 	bool horiz;
 	StepData sd = data[step], sd1;
+	gcpTheme *pTheme = pView->GetDoc ()->GetTheme ();
 	map<gcpRetrosynthesisStep *, gcpRetrosynthesisArrow *> *Arrows = step->GetArrows ();
 	map<gcpRetrosynthesisStep *, gcpRetrosynthesisArrow *>::iterator i, end = Arrows->end ();
 	for (i = Arrows->begin (); i != end; i++) {
@@ -330,38 +324,38 @@ static void AlignStep (map<Object*, StepData> &data, gcpRetrosynthesisStep *step
 			horiz = false;
 		if (horiz) {
 			if (x > 0)
-				dx = sd.r.x1 - sd.x + pView->GetArrowPadding () * pData->ZoomFactor;
+				dx = sd.r.x1 - sd.x + pTheme->GetArrowPadding ();
 			else
-				dx = sd.r.x0 - sd.x - pView->GetArrowPadding () * pData->ZoomFactor;
+				dx = sd.r.x0 - sd.x - pTheme->GetArrowPadding ();
 			dy = dx * y / x;
 		} else {
 			if (y > 0)
-				dy = sd.r.y1 - sd.y + pView->GetArrowPadding () * pData->ZoomFactor;
+				dy = sd.r.y1 - sd.y + pTheme->GetArrowPadding ();
 			else
-				dy = sd.r.y0 - sd.y - pView->GetArrowPadding () * pData->ZoomFactor;
+				dy = sd.r.y0 - sd.y - pTheme->GetArrowPadding ();
 			dx = dy * x / y;
 		}
-		x1 += x0 = (dx + sd.x) / pData->ZoomFactor - x0;
-		y1 += y0 = (dy + sd.y) / pData->ZoomFactor - y0;
+		x1 += x0 = (dx + sd.x) / pTheme->GetZoomFactor () - x0;
+		y1 += y0 = (dy + sd.y) / pTheme->GetZoomFactor () - y0;
 		(*i).second->Move (x0, y0);
 		pView->Update ((*i).second);
 		sd1 = data[(*i).first];
 		if (horiz) {
 			if (x > 0)
-				dx = sd1.x - sd1.r.x0 + pView->GetArrowPadding () * pData->ZoomFactor;
+				dx = sd1.x - sd1.r.x0 + pTheme->GetArrowPadding ();
 			else
-				dx = sd1.x - sd1.r.x1 +- pView->GetArrowPadding () * pData->ZoomFactor;
+				dx = sd1.x - sd1.r.x1 +- pTheme->GetArrowPadding ();
 			dy = dx * y / x;
 		} else {
 			if (y > 0)
-				dy = sd1.y - sd1.r.y0 + pView->GetArrowPadding () * pData->ZoomFactor;
+				dy = sd1.y - sd1.r.y0 + pTheme->GetArrowPadding ();
 			else
-				dy = sd1.y - sd1.r.y1 - pView->GetArrowPadding () * pData->ZoomFactor;
+				dy = sd1.y - sd1.r.y1 - pTheme->GetArrowPadding ();
 			dx = dy * x / y;
 		}
-		dx = x1 * pData->ZoomFactor - (sd1.x - dx);
-		dy = y1 * pData->ZoomFactor - (sd1.y - dy);
-		(*i).first->Move (dx / pData->ZoomFactor, dy / pData->ZoomFactor);
+		dx = x1 * pTheme->GetZoomFactor () - (sd1.x - dx);
+		dy = y1 * pTheme->GetZoomFactor () - (sd1.y - dy);
+		(*i).first->Move (dx / pTheme->GetZoomFactor (), dy / pTheme->GetZoomFactor ());
 		pView->Update ((*i).first);
 		sd1.r.x0 += dx;
 		sd1.r.x1 += dx;
@@ -377,6 +371,7 @@ static void AlignStep (map<Object*, StepData> &data, gcpRetrosynthesisStep *step
 void gcpRetrosynthesis::Align ()
 {
 	gcpDocument *pDoc = reinterpret_cast<gcpDocument *> (GetDocument ());
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpView *pView = pDoc->GetView ();
 	gcpWidgetData  *pData = reinterpret_cast<gcpWidgetData *> (g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data"));
 	/* Update the canvas if necessary */
@@ -392,7 +387,7 @@ void gcpRetrosynthesis::Align ()
 		if (pObj->GetType () == RetrosynthesisStepType) {
 			pData->GetObjectBounds (pObj, &sd.r);
 			sd.x = (sd.r.x0 + sd.r.x1) / 2.;
-			sd.y = pObj->GetYAlign () * pData->ZoomFactor;
+			sd.y = pObj->GetYAlign () * pTheme->GetZoomFactor ();
 			positions[pObj] = sd;
 		}
 		pObj = GetNextChild (i);
