@@ -26,13 +26,13 @@
 #include <math.h>
 #include "bondtool.h"
 #include "lib/settings.h"
-#include "lib/mendeleiev.h"
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/atom.h"
 #include "lib/bond.h"
 #include "lib/theme.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
+#include <glib/gi18n-lib.h>
 
 gcpBondTool::gcpBondTool (gcpApplication *App, string ToolId, unsigned nPoints): gcpTool (App, ToolId)
 {
@@ -196,7 +196,11 @@ void gcpBondTool::OnDrag()
 		GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		Object* pObject = NULL;
-		if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
+		if (pItem) {
+			pObject = (Object*) g_object_get_data(G_OBJECT(pItem), "object");
+			if (pObject == m_pObject || (pObject->GetType() == FragmentType) && dynamic_cast<gcpFragment*>(pObject)->GetAtom () == m_pObject)
+				return;
+		}
 		double dAngle;
 		m_pAtom = NULL;
 		if (MergeAtoms && pObject)
@@ -505,15 +509,12 @@ GtkWidget *gcpBondTool::GetPropertyPage ()
 	return glade_xml_get_widget (xml, "bond");
 }
 
-extern bool new_ui;
 void gcpBondTool::Activate ()
 {
-	if (new_ui) {
-		gcpDocument *pDoc = m_pApp->GetActiveDocument ();
-		gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
-		gtk_spin_button_set_value (m_AngleBtn, pDoc->GetBondAngle ());
-		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
-	}
+	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
+	gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
+	gtk_spin_button_set_value (m_AngleBtn, pDoc->GetBondAngle ());
+	gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
 }
 
 gcpUpBondTool::gcpUpBondTool(gcpApplication *App): gcpBondTool(App, "UpBond", 3)
