@@ -2,7 +2,7 @@
  * GChemPaint selection plugin
  * groupdlg.cc
  *
- * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +24,8 @@
 #include "groupdlg.h"
 #include "selectiontool.h"
 #include <lib/application.h>
+#include <lib/document.h>
+#include <lib/theme.h>
 #include <lib/widgetdata.h>
 
 static void on_align_toggled (gcpGroupDlg *dlg)
@@ -37,8 +39,12 @@ static void on_space_toggled (gcpGroupDlg *dlg)
 }
 
 gcpGroupDlg::gcpGroupDlg (gcpDocument *Doc, gcpGroup *group):
-	Dialog (Doc->GetApplication (), DATADIR"/gchempaint-unstable/ui/group.glade", "group")
+	Dialog (Doc->GetApplication (), DATADIR"/gchempaint-unstable/ui/group.glade", "group", Doc)
 {
+	if (!xml) {
+		delete this;
+		return;
+	}
 	m_Group = group;
 	m_Doc = Doc;
 	m_Data = (gcpWidgetData*) g_object_get_data (G_OBJECT (Doc->GetWidget ()), "data");
@@ -68,8 +74,9 @@ gcpGroupDlg::gcpGroupDlg (gcpDocument *Doc, gcpGroup *group):
 			gtk_widget_set_sensitive (GTK_WIDGET (padding_btn), false);
 		}
 	} else {
+		gcpTheme *pTheme = Doc->GetTheme ();
 		gtk_combo_box_set_active (align_box, 0);
-		gtk_spin_button_set_value (padding_btn, m_Data->ObjectPadding / m_Data->ZoomFactor);
+		gtk_spin_button_set_value (padding_btn, pTheme->GetObjectPadding () / pTheme->GetZoomFactor ());
 	}
 	g_signal_connect_swapped (align_btn, "toggled", G_CALLBACK (on_align_toggled), this);
 	g_signal_connect_swapped (space_btn, "toggled", G_CALLBACK (on_space_toggled), this);
