@@ -91,12 +91,14 @@ static bool filter_attribute (PangoAttribute *attribute, gcpTextTool *tool)
 	return false;
 }
 
-gcpTextTool::gcpTextTool (gcpApplication* App, string Id): gcpTool (App, Id)
+gcpTextTool::gcpTextTool (gcpApplication* App, string Id):
+	gcpTool (App, Id),
+	m_FamilyList (NULL)
 {
 	m_Active = NULL;
 	m_bUndo = true;
 	m_CurNode = m_InitNode = NULL;
-	m_Bold = m_Italic = m_Strikethrough = m_Subscript = m_Superscript = false;
+	m_Strikethrough = false;
 	m_FontDesc = NULL;
 	gcpTheme *pTheme = ThemeManager.GetTheme ("Default");
 	m_FamilyName = pTheme->GetTextFontFamily ();
@@ -106,7 +108,8 @@ gcpTextTool::gcpTextTool (gcpApplication* App, string Id): gcpTool (App, Id)
 	m_Variant = pTheme->GetFontVariant ();
 	m_Size = pTheme->GetFontSize ();
 	m_Underline = PANGO_UNDERLINE_NONE;
-	m_Scalable = false; // force sizes list creation
+	m_Rise = 0;
+	m_Color = RGBA_BLACK;
 	m_SelSignal = 0;
 }
 
@@ -209,17 +212,48 @@ bool gcpTextTool::OnEvent (GdkEvent* event)
 					return true;
 				case GDK_plus:
 				case GDK_dead_circumflex:
-					BuildAttributeList ();
+					if (m_Rise == 0) {
+						m_Size = m_Size * 2 / 3;
+						m_Rise = m_Size / PANGO_SCALE;
+					} else if (m_Rise < 0)
+						m_Rise = m_Size / PANGO_SCALE;
+					else {
+						m_Size = m_Size * 3 / 2;
+						m_Rise = 0;
+					}
+					g_signal_handler_block (m_RiseButton, m_RiseSignal);
+					gtk_spin_button_set_value (m_RiseButton, m_Rise);
+					m_Rise *= PANGO_SCALE;
+					g_signal_handler_unblock (m_RiseButton, m_RiseSignal);
+					SetSizeFull (true);
 					return true;
 				case GDK_equal:
 				case GDK_underscore:
+					if (m_Rise == 0) {
+						m_Size = m_Size * 2 / 3;
+						m_Rise = - m_Size / PANGO_SCALE / 2;
+					} else if (m_Rise > 0)
+						m_Rise = - m_Size / PANGO_SCALE/ 2;
+					else {
+						m_Size = m_Size * 3 / 2;
+						m_Rise = 0;
+					}
+					g_signal_handler_block (m_RiseButton, m_RiseSignal);
+					gtk_spin_button_set_value (m_RiseButton, m_Rise);
+					m_Rise *= PANGO_SCALE;
+					g_signal_handler_unblock (m_RiseButton, m_RiseSignal);
+					SetSizeFull (true);
+					return true;
+				case GDK_space: {
+					GnomeCanvasPango* saved = m_Active;
+					m_Active = NULL;
+					UpdateAttributeList ();
+					m_Active = saved;
 					BuildAttributeList ();
 					return true;
-				case GDK_space:
-					BuildAttributeList ();
-					return true;
+				}
 				default:
-					return false;
+					break;
 				}
 			}
 			if (!g_utf8_validate (((GdkEventKey*) event)->string, -1, NULL)) {
@@ -248,14 +282,8 @@ bool gcpTextTool::OnEvent (GdkEvent* event)
 
 void gcpTextTool::Activate ()
 {
-	gcpTheme *pTheme = m_pApp->GetActiveDocument ()->GetTheme ();
-	m_FamilyName = pTheme->GetTextFontFamily ();
-#warning FIXME: use Text variants for following lines
-	m_Style = pTheme->GetFontStyle ();
-	m_Weight = pTheme->GetFontWeight ();
-	m_Stretch = pTheme->GetFontStretch ();
-	m_Variant = pTheme->GetFontVariant ();
-	m_Size = pTheme->GetFontSize ();
+	if (!m_Active)
+		UpdateAttributeList ();
 }
 
 bool gcpTextTool::Deactivate ()
@@ -602,7 +630,7 @@ void gcpTextTool::OnGetData (GtkClipboard *clipboard, GtkSelectionData *selectio
 
 void gcpTextTool::UpdateAttributeList ()
 {
-	if (!m_Active)
+	if (!m_FamilyList)
 		return;
 	gcpTheme *pTheme = m_pApp->GetActiveDocument ()->GetTheme ();
 	m_FamilyName = pTheme->GetTextFontFamily ();
@@ -614,14 +642,14 @@ void gcpTextTool::UpdateAttributeList ()
 	m_Rise = 0;
 	m_Underline = PANGO_UNDERLINE_NONE;
 	m_Strikethrough = false;
-	PangoLayout *layout;
-	g_object_get (m_Active, "layout", &layout, NULL);
-	PangoAttrList *l = pango_layout_get_attributes (layout);
-	if (!l)
-		return;
-	pango_attr_list_filter (l, (PangoAttrFilterFunc) filter_attribute, this);
-	if (m_Rise != 0)
-		m_Size *= 2 / 3;
+	m_Color = RGBA_BLACK;
+	if (m_Active) {
+		PangoLayout *layout;
+		g_object_get (m_Active, "layout", &layout, NULL);
+		PangoAttrList *l = pango_layout_get_attributes (layout);
+		if (l)
+			pango_attr_list_filter (l, (PangoAttrFilterFunc) filter_attribute, this);
+	}
 	// select the found face
 	GtkTreeIter iter;
 	char const *buf;
@@ -644,6 +672,9 @@ void gcpTextTool::UpdateAttributeList ()
 	g_signal_handler_block (m_StrikethroughBtn, m_StrikethroughSignal);
 	gtk_toggle_button_set_active (m_StrikethroughBtn, m_Strikethrough);
 	g_signal_handler_unblock (m_StrikethroughBtn, m_StrikethroughSignal);
+	g_signal_handler_block (m_RiseButton, m_RiseSignal);
+	gtk_spin_button_set_value (m_RiseButton, m_Rise / PANGO_SCALE);
+	g_signal_handler_unblock (m_RiseButton, m_RiseSignal);
 	g_signal_handler_block (m_ColorSelector, m_ForeSignal);
 	go_color_selector_set_color (m_ColorSelector, m_Color);
 	g_signal_handler_unblock (m_ColorSelector, m_ForeSignal);
@@ -663,6 +694,7 @@ void gcpTextTool::BuildAttributeList ()
 	pango_attr_list_insert (l, pango_attr_size_new (m_Size));
 	pango_attr_list_insert (l, pango_attr_underline_new (m_Underline));
 	pango_attr_list_insert (l, pango_attr_strikethrough_new (m_Strikethrough));
+	pango_attr_list_insert (l, pango_attr_rise_new (m_Rise));
 	pango_attr_list_insert (l, pango_attr_foreground_new (UINT_RGBA_R (m_Color) * 0x101, UINT_RGBA_G (m_Color) * 0x101, UINT_RGBA_B (m_Color) * 0x101));
 	gnome_canvas_pango_set_insert_attrs (m_Active, l);
 	m_Dirty = false;
@@ -716,6 +748,11 @@ static void on_fore_color_changed (GOSelector *sel, gcpTextTool *tool)
 	tool->OnForeColorChanged (go_color_selector_get_color (sel, &auto_color));
 }
 
+static void on_rise_changed (GtkSpinButton *btn, gcpTextTool *tool)
+{
+	tool->OnPositionChanged (gtk_spin_button_get_value_as_int (btn));
+}
+
 /* These are what we use as the standard font sizes, for the size list.
  */
 static const guint16 font_sizes[] = {
@@ -815,7 +852,10 @@ GtkWidget *gcpTextTool::GetPropertyPage ()
 	m_UnderlineSignal = g_signal_connect (G_OBJECT (m_UnderlineBox), "changed", G_CALLBACK (on_underline_changed), this);
 	m_StrikethroughBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "strikethrough"));
 	m_StrikethroughSignal = g_signal_connect (G_OBJECT (m_StrikethroughBtn), "toggled", G_CALLBACK (on_strikethrough_toggled), this);
+	m_RiseButton = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "rise"));
+	m_RiseSignal = g_signal_connect (G_OBJECT (m_RiseButton), "value-changed", G_CALLBACK (on_rise_changed), this);
 	m_ColorSelector = GO_SELECTOR (go_color_selector_new (RGBA_BLACK, RGBA_BLACK, "fore"));
+	go_color_selector_set_allow_alpha (m_ColorSelector, false);
 	m_ForeSignal = g_signal_connect (G_OBJECT (m_ColorSelector), "activate", G_CALLBACK (on_fore_color_changed), this);
 	gtk_widget_show (GTK_WIDGET (m_ColorSelector));
 	gtk_table_attach (GTK_TABLE (glade_xml_get_widget (xml, "table2")), GTK_WIDGET (m_ColorSelector), 1, 2, 0, 1, (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
@@ -1043,8 +1083,9 @@ void gcpTextTool::OnStriketroughToggled (bool strikethrough)
 	}
 }
 
-void gcpTextTool::OnPositionChanged (unsigned position)
+void gcpTextTool::OnPositionChanged (int position)
 {
+	m_Rise = position * PANGO_SCALE;
 	BuildAttributeList ();
 }
 
